function [y_pre,optimal_lag] = AR_DI_forecast_new(Y,h,lag_max,F)
% Y is T*1 vector of univariate time series
% F is a T*r matrix of PCA factors a function that generate a h-step-ahead forecast of y based on AR models, where lag_max is the maximum length of lags and, and lag order is selected using BIC

T=length(Y); 
%T_sample=T-h; % the size of data used to estimate the model

Y_h=Y(lag_max+h:T);

n=length(Y_h);

X=[];
for j=lag_max:-1:1
    X=[X Y(j:j+n-1)  ];
end
X=[ones(n,1) X];

% calculate the SSRs for each k=1:lag_max
SSR=[];
for k=1:lag_max
    dum_X = [X(:,1:k+1) F(lag_max:T-h, :) ];
    PX= dum_X* inv(dum_X'*dum_X)* (dum_X');
    Resid = (eye(n) - PX)*Y_h;
    SSR=[SSR; Resid'*Resid];
end

% use BIC to choose the optimal lag length
K=1:k;
BIC = log(SSR/n)+ (K'+1)*log(n)/n;
[dum, optimal_lag] = min(BIC);  

% generate the forecast
dumX=[X(:,1:optimal_lag+1) F(lag_max:T-h, :)];
beta = inv(dumX'*dumX)* (dumX')*Y_h;
y_pre = (beta')*[1;Y(T:-1:T-optimal_lag+1);F(T,:)'];













