clear
close all
clc


csv_in='2020-01.csv';
DEMEAN=2;
jj=2;
kmax=8;


%% PART 1: LOAD AND LABEL DATA

% Load data from CSV file
dum=importdata(csv_in,',');

% Variable names
series=dum.textdata(1,2:end);

% Transformation numbers
tcode=dum.data(2,:);

% Raw data 
% QUARTERLY DATA FROM 1959Q1 TO 2019Q4
rawdata=dum.data(3:end,:);
dates = (1959:0.25:2019.75)';

% T = number of QUARTERS in sample
T=size(dates,1);
rawdata=rawdata(1:T,:);

%% PART 2: PROCESS DATA

% Transform raw data to be stationary using auxiliary function
% prepare_missing()
yt=prepare_missing(rawdata,tcode);

% Reduce sample to usable dates: remove first two months because some
% series have been first differenced
yt=yt(5:T-2,:);
dates=dates(5:T-2,:);         % 1960q1 to 2019q2

X_raw = yt(:,~any(isnan(yt),1));   % 211 series without missing value

%X_raw(:,58)=[];
[T,N]=size(X_raw);  % T=238 N=211

% standardize the data
 m=mean(X_raw);
 v=var(X_raw);
 X=(X_raw-ones(T,1)*m)./(ones(T,1)*sqrt(v));

save FRED_Q_ALLDATA.mat X_raw X;    %  1960q1 to 2019q2        T=238 N=211

%% PART 3: Estimating the Number of Factors

L=min(N,T);
kmax=8;
tol=0.001;
tau=0.99;
pan=(1/L)^(1/3);

% PC_p1 of BN2002
[PC_1,dum1,dum2,dum3] = nbpiid(X,kmax,1,0); % PC_p1 of BN2002

TABLE3_1=[];

% r_rank by rank-minimization 
for tau=[0.01 0.05 0.1 0.25 0.5 0.75 0.9 0.95 0.99]
[Fhat,Lhat]=IQR(X,kmax,tol,tau);
dum=Lhat'*Lhat/N;
r_rank=sum( diag(dum)>pan*dum(1,1));
TABLE3_1=[TABLE3_1;tau r_rank];
end

% results for the number of factors
% tau=0.01   -----1
% tau=0.05  ------1
% tau=0.10 ---- 2
% tau=0.25 ---- 4
% tau=0.50 --- 5
% tau=0.75  --- 5
% tau=0.90 --- 2
% tau=0.95 --- 1
% tau=0.99 --- 1


%%   PART 4: estimating the facotrs

 [Fhat_PCA,dum2,dum3] = svd(X*X');
  Fhat_PCA=Fhat_PCA(:,1:8)*sqrt(T);

  tau=0.01;
 [Fhat_1,Lhat_10]=IQR(X,1,tol,tau); 
 
 tau=0.05;
 [Fhat_5,Lhat_10]=IQR(X,1,tol,tau);  
  
 tau=0.1;
 [Fhat_10,Lhat_10]=IQR(X,2,tol,tau);  

 tau=0.25;
 [Fhat_25,Lhat_25]=IQR(X,4,tol,tau);  
 
 tau=0.5;
[Fhat_50,Lhat_50]=IQR(X,5,tol,tau);  
 
tau=0.75;
[Fhat_75,Lhat_75]=IQR(X,5,tol,tau);  

tau=0.9;
[Fhat_90,Lhat_90]=IQR(X,2,tol,tau);   

 tau=0.95;
 [Fhat_95,Lhat_10]=IQR(X,1,tol,tau); 
 
 tau=0.99;
 [Fhat_99,Lhat_10]=IQR(X,1,tol,tau); 
 
 
 save FRED_Q_ALL_FACTOR.mat Fhat_PCA Fhat_1 Fhat_5 Fhat_10 Fhat_25 Fhat_50 Fhat_75 Fhat_90 Fhat_95 Fhat_99;
 

