clear all;
clc;

tic 

load FRED_Q_ALLDATA.mat     % FRED data, 211 macro series, 1960Q1 to 2019Q2, T=238
CPI=csvread('CPI.csv',1,1);
dates_CPI=(1947:1/12:2020)';
CPI=CPI(145:870); % 1959M1 to 2019M6
CPI_Q=[];     % CPI from 1959Q1 to 2019Q2 
for s=0:241
    dum=sum(CPI(3*s+1:3*s+3));
    CPI_Q=[CPI_Q;dum];
end
inflation=log(CPI_Q(5:end)) - log(CPI_Q(1:end-4));
INFLATION=100*inflation; % 1960Q1 to 2019Q2

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
GDP=INFLATION; % quaterly real Inflation rate from 1960Q1 to 2019Q2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

dates_Q=(1960:1/4:2019+1/4)';

%% forecasting GDP

% real time simulation starting from 1990Q1
enddate=238; % 190 -- 2007Q2  238 -- 2019Q2
startdate=121; % 121 -- 1990Q1
GDP_true=GDP(startdate:enddate);     

h=1; % one-step ahead
lag_max=4;
kmax=5; % maximum number of PCA factors

%% AR forecast + BIC 
Y_pre_AR=[];
for j=startdate:enddate
[dum1,dum2]=AR_forecast_new(GDP(j-h-119:j-h),h,lag_max);
Y_pre_AR=[Y_pre_AR;dum1];
end

% calculating MSE
MSE_AR_PRE=mean((GDP_true(1:70)-Y_pre_AR(1:70)).^2);
MSE_AR_POST=mean((GDP_true(71:118)-Y_pre_AR(71:118)).^2);
MSE_AR=mean((GDP_true-Y_pre_AR).^2);


%% AR+DI forecast + BIC 
Y_pre_DI=[];
for j=startdate:enddate
    
   % estimating PCA factors 
 Z=X(j-h-119:j-h,:);
 [TT,NN]=size(Z);
 [PC_1,dum1,dum2,dum3] = nbpiid(Z,kmax,1,2);
 [FPCA,dum2,dum3] = svd(Z*Z');
  FPCA=FPCA(:,1:PC_1)*sqrt(TT); 

  % forecasting
[dum1,dum2]=AR_DI_forecast_new(GDP(j-h-119:j-h),h,lag_max,FPCA);
Y_pre_DI=[Y_pre_DI;dum1];
end

% calculating MSE
MSE_AR_DI_PRE=mean((GDP_true(1:70)-Y_pre_DI(1:70)).^2)/MSE_AR_PRE;
MSE_AR_DI_POST=mean((GDP_true(71:118)-Y_pre_DI(71:118)).^2)/MSE_AR_POST;
MSE_AR_DI=mean((GDP_true-Y_pre_DI).^2)/MSE_AR;

%% AR+QDI forecast + BIC 
Y_pre_Q90=[];
Y_pre_Q99=[];
Y_pre_Q90_Q99=[];
for j=startdate:enddate
    
% estimating PCA factors 
 Z=X(j-h-119:j-h,:);
 [TT,NN]=size(Z);

 [PC_1,dum1,dum2,dum3] = nbpiid(Z,kmax,1,2);
 [FPCA,dum2,dum3] = svd(Z*Z');
  FPCA=FPCA(:,1:PC_1)*sqrt(TT); 
  
% estimating quantile factors
tol=0.001;
[QF_90,Lhat_90]=IQR(Z,2,tol,0.9); 
[QF_99,Lhat_10]=IQR(Z,1,tol,0.99); 

% forecasting 
[dum1,dum]=AR_DI_forecast_new(GDP(j-h-119:j-h),h,lag_max,[FPCA QF_90(:,1)]);
Y_pre_Q90=[Y_pre_Q90;dum1];

[dum1,dum]=AR_DI_forecast_new(GDP(j-h-119:j-h),h,lag_max,[FPCA QF_99]);
Y_pre_Q99=[Y_pre_Q99;dum1];

[dum1,dum]=AR_DI_forecast_new(GDP(j-h-119:j-h),h,lag_max,[FPCA QF_90(:,1) QF_99]);
Y_pre_Q90_Q99=[Y_pre_Q90_Q99;dum1];
end

% calculating MSE
MSE_AR_Q90=mean((GDP_true-Y_pre_Q90).^2)/MSE_AR;
MSE_AR_Q99=mean((GDP_true-Y_pre_Q99).^2)/MSE_AR;
MSE_AR_Q90_Q99=mean((GDP_true-Y_pre_Q90_Q99).^2)/MSE_AR;

MSE_AR_Q90_PRE=mean((GDP_true(1:70)-Y_pre_Q90(1:70)).^2)/MSE_AR_PRE;
MSE_AR_Q99_PRE=mean((GDP_true(1:70)-Y_pre_Q99(1:70)).^2)/MSE_AR_PRE;
MSE_AR_Q90_Q99_PRE=mean((GDP_true(1:70)-Y_pre_Q90_Q99(1:70)).^2)/MSE_AR_PRE;

MSE_AR_Q90_POST=mean((GDP_true(71:118)-Y_pre_Q90(71:118)).^2)/MSE_AR_POST;
MSE_AR_Q99_POST=mean((GDP_true(71:118)-Y_pre_Q99(71:118)).^2)/MSE_AR_POST;
MSE_AR_Q90_Q99_POST=mean((GDP_true(71:118)-Y_pre_Q90_Q99(71:118)).^2)/MSE_AR_POST;

toc 
%% Results: Table 4 Lower Panel 

TABLE4_LOWER = zeros(5,6);
% Row 1
TABLE4_LOWER(1,1) = MSE_AR_PRE;
TABLE4_LOWER(1,2)=1;
TABLE4_LOWER(1,3) = MSE_AR_POST;
TABLE4_LOWER(1,4)=1;
TABLE4_LOWER(1,5) = MSE_AR;
TABLE4_LOWER(1,6)=1;
% Row 2
TABLE4_LOWER(2,1) =TABLE4_LOWER(1,1)*MSE_AR_DI_PRE;
TABLE4_LOWER(2,2)=MSE_AR_DI_PRE;
TABLE4_LOWER(2,3) = TABLE4_LOWER(1,3)*MSE_AR_DI_POST;
TABLE4_LOWER(2,4)=MSE_AR_DI_POST;
TABLE4_LOWER(2,5) = TABLE4_LOWER(1,5)*MSE_AR_DI;
TABLE4_LOWER(2,6)=MSE_AR_DI;
% Row 3
TABLE4_LOWER(3,1) =TABLE4_LOWER(1,1)*MSE_AR_Q90_PRE;
TABLE4_LOWER(3,2)=MSE_AR_Q90_PRE;
TABLE4_LOWER(3,3) = TABLE4_LOWER(1,3)*MSE_AR_Q90_POST;
TABLE4_LOWER(3,4)=MSE_AR_Q90_POST;
TABLE4_LOWER(3,5) = TABLE4_LOWER(1,5)*MSE_AR_Q90;
TABLE4_LOWER(3,6)=MSE_AR_Q90;
% Row 4
TABLE4_LOWER(4,1) =TABLE4_LOWER(1,1)*MSE_AR_Q99_PRE;
TABLE4_LOWER(4,2)=MSE_AR_Q99_PRE;
TABLE4_LOWER(4,3) = TABLE4_LOWER(1,3)*MSE_AR_Q99_POST;
TABLE4_LOWER(4,4)=MSE_AR_Q99_POST;
TABLE4_LOWER(4,5) = TABLE4_LOWER(1,5)*MSE_AR_Q99;
TABLE4_LOWER(4,6)=MSE_AR_Q99;
% Row 5
TABLE4_LOWER(5,1) =TABLE4_LOWER(1,1)*MSE_AR_Q90_Q99_PRE;
TABLE4_LOWER(5,2)=MSE_AR_Q90_Q99_PRE;
TABLE4_LOWER(5,3) = TABLE4_LOWER(1,3)*MSE_AR_Q90_Q99_POST;
TABLE4_LOWER(5,4)=MSE_AR_Q90_Q99_POST;
TABLE4_LOWER(5,5) = TABLE4_LOWER(1,5)*MSE_AR_Q90_Q99;
TABLE4_LOWER(5,6)=MSE_AR_Q90_Q99;




















