clear all;
clc;


tic 


%% load the inflation data 

load FRED_Q_ALLDATA.mat     % FRED data, 211 macro series, 1960Q1 to 2019Q2, T=238


%GDP=csvread('GDP.csv',1,1);

CPI=csvread('CPI.csv',1,1);
dates_CPI=(1947:1/12:2020)';
CPI=CPI(145:870); % 1959M1 to 2019M6
CPI_Q=[];     % CPI from 1959Q1 to 2019Q2 
for s=0:241
    dum=sum(CPI(3*s+1:3*s+3));
    CPI_Q=[CPI_Q;dum];
end
inflation=log(CPI_Q(5:end)) - log(CPI_Q(1:end-4));
INFLATION=100*inflation; % 1960Q1 to 2019Q2




GDP=INFLATION; % quaterly real Inflation rate from 1960Q1 to 2019Q2



dates_Q=(1960:1/4:2019+1/4)';

%% paramter specification

h=1;   %forecasting horizon
lag=4; % number of lags 
kmax=5;
roll=120-h; % length of rolling windows
tol=0.001;


%% forecasting setup

enddate=238; % 190 -- 2007Q2  238 -- 2019Q2
startdate=121; % 121 -- 1990Q1
GDP_true=GDP(startdate:enddate);    % the true GDP to forecast 1990Q1 to 2019Q2

%% AR forecast 
Y_pre_AR_5=[];
Y_pre_AR_25=[];
Y_pre_AR_75=[];
Y_pre_AR_95=[];

for j=startdate:enddate
   
    tau=0.05;
   Y_pre_AR_5=[Y_pre_AR_5; AR_QuantileForecast( GDP(j-h-roll:j-h),h,lag,tau )] ;
   
    tau=0.25;
   Y_pre_AR_25=[Y_pre_AR_25; AR_QuantileForecast( GDP(j-h-roll:j-h),h,lag,tau )] ;
      
   tau=0.75;
   Y_pre_AR_75=[Y_pre_AR_75; AR_QuantileForecast( GDP(j-h-roll:j-h),h,lag,tau )] ;
      
   tau=0.95;
   Y_pre_AR_95=[Y_pre_AR_95; AR_QuantileForecast( GDP(j-h-roll:j-h),h,lag,tau )] ;
end

%% AR + QF  forecast 
Y_pre_AR_QF_5=[];
Y_pre_AR_QF_25=[];
Y_pre_AR_QF_75=[];
Y_pre_AR_QF_95=[];

for j=startdate:enddate
   
     Z=X(j-h-roll:j-h,:);
    
    tau=0.05;
    [F,dum]=IQR(Z,1,tol,tau);
   Y_pre_AR_QF_5=[Y_pre_AR_QF_5; AR_Factor_QuantileForecast( GDP(j-h-roll:j-h),h,lag,tau,F )] ;
   
    tau=0.25;
    [F,dum]=IQR(Z,5,tol,tau);
   Y_pre_AR_QF_25=[Y_pre_AR_QF_25; AR_Factor_QuantileForecast( GDP(j-h-roll:j-h),h,lag,tau,F )] ;
      
   tau=0.75;
   [F,dum]=IQR(Z,5,tol,tau);
   Y_pre_AR_QF_75=[Y_pre_AR_QF_75; AR_Factor_QuantileForecast( GDP(j-h-roll:j-h),h,lag,tau,F )] ;
      
   tau=0.95;
   [F,dum]=IQR(Z,1,tol,tau);
   Y_pre_AR_QF_95=[Y_pre_AR_QF_95; AR_Factor_QuantileForecast( GDP(j-h-roll:j-h),h,lag,tau,F )] ;
end



%% calculating Predictive Score

addpath('azzalini')

PS_AR=[];
PS_AR_QF=[];
FIT_AR=[];
FIT_AR_QF=[]
for s=1:118
    
    [lc,sc,sh,df] = QuantilesInterpolation([Y_pre_AR_5(s) Y_pre_AR_25(s) Y_pre_AR_75(s) Y_pre_AR_95(s) ], [0.05 0.25 0.75 0.95]);
    dum=[lc;sc;sh;df];
    PS_AR=[PS_AR;dskt(GDP_true(s),lc,sc,sh,df ) ];
    FIT_AR=[FIT_AR dum ];
    
    [lc,sc,sh,df] = QuantilesInterpolation([Y_pre_AR_QF_5(s) Y_pre_AR_QF_25(s) Y_pre_AR_QF_75(s) Y_pre_AR_QF_95(s) ], [0.05 0.25 0.75 0.95]);
    dum=[lc;sc;sh;df];
    PS_AR_QF=[PS_AR_QF;dskt(GDP_true(s),lc,sc,sh,df ) ];
    FIT_AR_QF=[FIT_AR_QF dum ];
end

years=(1990:1/4:2019+1/4)';


toc
save Inflation_DensityPrediction_4lag_1year.mat PS_AR PS_AR_QF Y_pre_AR_5 Y_pre_AR_25 Y_pre_AR_75 Y_pre_AR_95 Y_pre_AR_QF_5 Y_pre_AR_QF_25 Y_pre_AR_QF_75 Y_pre_AR_QF_95 FIT_AR FIT_AR_QF



%% plot the scores

load Inflation_DensityPrediction_4lag_1year.mat;

plot(years,PS_AR,'--b','LineWidth',1)
hold on
grid on
plot(years,PS_AR_QF,'r','LineWidth',1)
legend({'AR Benchmark','AR + Quantile Factors'},'FontSize',12)
set(gca,'FontSize',12)
xAX = get(gca,'XAxis');
set(xAX,'FontSize', 12)
ylim([0,3.5])
title('Predictive Scores of 1-Year-Ahead Density Forecasts for Inflation')