
clear all;

% random number generator seed
rng(2019);

%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% To obtain the results in Table 1, you need to specify the values of "N,T,nu"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% set the sample size
N=200;
T=100;
% degrees of freedom for the error distribution
nu=3;

% the number of repetitions
rep=1000;

% parameters for the QFM
tau=0.5;
tol=0.001;
kmax=8;

% parameters for the rank estimator
L=min(N,T);
pan=(1/L)^(1/3);

% AR coeffcients for the factors 
b1=0.8;
b2=0.5;
b3=0.2;

% vectors to store the results
 R2_KEN=[];
 R2_IQR=[];

RHAT=[];



parfor s=1:rep

rng(s,'twister');    

%% DGP
u=randn(T,3);
F=zeros(T,3);
F(1,:)=u(1,:);
for h=2:T;
    F(h,1)=b1*F(h-1,1)+u(h,1);
    F(h,2)=b2*F(h-1,2)+u(h,2);
    F(h,3)=b3*F(h-1,3)+u(h,3);
end

Lambda=randn(N,3);
e=trnd(nu,T,N); % student's t distrubiton with nu degrees of freedom
X=F*Lambda'+e;


%% estimating the number of factors using IQR
 [Fhat,Lhat]=IQR(X,kmax,tol,tau);
 dum=Lhat'*Lhat/N;
 rhat=sum( diag(dum)>pan*dum(1,1));
 RHAT=[RHAT;rhat];

%% estimating the 3 factors using IQR
  [Fhat_IQR,dum]=IQR(X,3,tol,tau);

%% Estimator based on Spatial Kendall's tau
K=zeros(N,N);
for p=2:T
    for q=1:p-1
        yp=X(p,:)';
        yq=X(q,:)';
        K=K+ ( (yp-yq)*(yp-yq)' )/( (yp-yq)'*(yp-yq) );
    end
end
K=K*2/N/(N-1);
[dum1,dum2]=svd(K);
L_KEN=dum1(:,1:3)*sqrt(N);
Fhat_KEN=X*L_KEN/N;
  
%% calculating the R2
r2_ken=[ rsquare_a(F(:,1),Fhat_KEN ) rsquare_a(F(:,2),Fhat_KEN ) rsquare_a(F(:,3),Fhat_KEN )  ];
r2_iqr=[ rsquare_a(F(:,1),Fhat_IQR ) rsquare_a(F(:,2),Fhat_IQR ) rsquare_a(F(:,3),Fhat_IQR )  ];
R2_KEN=[R2_KEN;r2_ken];
R2_IQR=[R2_IQR;r2_iqr];

end

%% final results
Mean_rhat=mean(RHAT) % column 2 of Table 1
frequencey = mean(RHAT==3) % column 3 of Table 1
Mean_R2_KEN=mean(R2_KEN) % columns 4 to 6 of Table 1
Mean_R2_IQR=mean(R2_IQR) % columns 7 to 9 of Table 1

