
clear all;

% set your own random number generator seed
rng(2019);

%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% To obtain the results in Table 2, you need to specify the values of "N,T"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set the sample size
 N=50;
 T=50;

% the number of repetitions
rep=1000;

% parameters for the QFM 
tol=0.005;
kmax=8;
L=min(N,T);
pan=(1/L)^(1/3);

% AR coeffcients for the factors
b1=0.8;
b2=0.5;

% autocorrelation of erorr
beta=0.2; 
% cross sectional correlation of error
rho=0.2; 

% store the results
RHAT_25=[];
RHAT_50=[];
RHAT_75=[];
R2_25=[];
R2_50=[];
R2_75=[];

parfor s=1:rep

    rng(s,'twister');    

    %% generate mean factors
u=randn(T,2);
F=zeros(T,2);
F(1,:)=u(1,:);
for h=2:T;
    F(h,1)=b1*F(h-1,1)+u(h,1);
    F(h,2)=b2*F(h-1,2)+u(h,2);
end
Lambda=randn(N,2);

%% generate errors
v=trnd(3,T,N); % errors generated as student's t distribution with 3 degrees of freedom

%% generate quantile factors
G=randn(T,1);
G=abs(G);
Gamma= rand(N,1)+1;

U=(G*Gamma').*v;
X=F*Lambda'+U;

%% tau=0.25
[Fhat,Lhat]=IQR(X,kmax,tol,0.25);
dum=Lhat'*Lhat/N;
rhat=sum( diag(dum)>pan*dum(1,1));
RHAT_25=[RHAT_25;rhat];

[Fhat,Lhat]=IQR(X,rhat,tol,0.25);

r2=[rsquare_a(F(:,1),Fhat ) rsquare_a(F(:,2),Fhat ) rsquare_a(G,Fhat)];
R2_25=[R2_25;r2];

%% tau=0.5
[Fhat,Lhat]=IQR(X,kmax,tol,0.5);
dum=Lhat'*Lhat/N;
rhat=sum( diag(dum)>pan*dum(1,1));
RHAT_50=[RHAT_50;rhat];

[Fhat,Lhat]=IQR(X,rhat,tol,0.5);

r2=[rsquare_a(F(:,1),Fhat ) rsquare_a(F(:,2),Fhat ) rsquare_a(G,Fhat)];
R2_50=[R2_50;r2];

%% tau=0.75
[Fhat,Lhat]=IQR(X,kmax,tol,0.75);
dum=Lhat'*Lhat/N;
rhat=sum( diag(dum)>pan*dum(1,1));
RHAT_75=[RHAT_75;rhat];

[Fhat,Lhat]=IQR(X,rhat,tol,0.75);

r2=[rsquare_a(F(:,1),Fhat ) rsquare_a(F(:,2),Fhat ) rsquare_a(G,Fhat)];
R2_75=[R2_75;r2];

end

%% final resuls for M2 in Table 2
rhat_mean_25=mean(RHAT_25)
R2_mean_25=mean(R2_25)
rhat_mean_50=mean(RHAT_50)
R2_mean_50=mean(R2_50)
rhat_mean_75=mean(RHAT_75)
R2_mean_75=mean(R2_75)






