
clear all;

rng(2019);

%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% To obtain the results in Table S.1, you need to specify the values of "N,T"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set the sample size
 N=200;
 T=200;

% number of repetitions 
rep=1000;

% kmax in IC_P1
kmax=8;

%  AR coefficients of the factors
b1=0.8;
b2=0.5;


beta=0.2; % autocorrelation of erorrs
rho=0.2; % cross sectional correlation of error
J=3;

% store the results
IC=[];
R2_PCA=[];

for s=1:rep
%% generate mean factors
u=randn(T,2);
F=zeros(T,2);
F(1,:)=u(1,:);

for h=2:T;
    F(h,1)=b1*F(h-1,1)+u(h,1);
    F(h,2)=b2*F(h-1,2)+u(h,2);
end
Lambda=randn(N,2);

%% generate errors
v=randn(N,T);
e=zeros(N,T);
e(:,1)=v(:,1);
%two ways of correlations
for t=2:T
    for j=1:N
        if j<=J && j>1
            e(j,t)=beta*e(j,t-1)+v(j,t)+rho*( sum(v(1:j-1,t))+sum(v(j+1:j+J,t) ));
        elseif j>N-J && J< N
            e(j,t)=beta*e(j,t-1)+v(j,t)+rho*(sum(v(j+1:N,t))+sum(v(j-J:j-1,t))  );
        elseif j==1
            e(j,t)=beta*e(j,t-1)+v(j,t)+rho* sum(v(j+1:j+J,t) );
        elseif j==N
            e(j,t)=beta*e(j,t-1)+v(j,t)+rho*sum(v(j-J:j-1,t))  ;
        else
            e(j,t)=beta*e(j,t-1)+v(j,t)+rho*(sum(v(j+1:j+J,t))+sum(v(j-J:j-1,t))  );
        end
    end
end
 e=e';

%% generate quantile factors
G=randn(T,1);
G=abs(G);
Gamma= rand(N,1)+1;
U=(G*Gamma').*e;
X=F*Lambda'+U;

%% estimating the number of factors using the IC of BN2002
[ic,dum1,dum2] = nbplog(X,kmax,1,0); % IC_p1 of BN2002 
IC=[IC;ic];

%% estimating the factors using PCA
Fhat_PCA=dum1(:,1:ic);

%% calculating the R2
r2_pca=[ rsquare_a(F(:,1),Fhat_PCA ) rsquare_a(F(:,2),Fhat_PCA ) rsquare_a(G,Fhat_PCA )  ];
R2_PCA=[R2_PCA;r2_pca];

end

%% final results: M3 in Table S.1 of the online appendix
IC_mean=mean(IC)
R2_mean=mean(R2_PCA)





