


clear all;

% set your own random number generator seed
%rng(2019);

%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% To obtain the results in Table 2, you need to specify the values of "N,T,h,b"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% sample size
N=50;
T=50;
% bandwidth parameters
h=0.5; 
b=0.5; 

% number of repetitions
rep=1000;

% other parameters
bound=25;
tau=0.25;

% generate the factors and loadings
lambda=randn(N,1);
F=rand(T,1)+1;
F=F/sqrt(F'*F/T);
lambda_tau=lambda+norminv(tau);

% store the results
Lambda1=[];
F_standard=[];
VAR_ESTIMATE =[];

parfor s=1:rep
    
%% DGP generte the errors
epsi=randn(T,N);
e=epsi.*(F*ones(1,N));
X=F*lambda'+e;

%% Smooth Iteration
F0=zeros(T,1);
F1=ones(T,1);

m=0;
while max(abs(F1-F0))>0.005 && m<3000    
    lambda1=[];
    for i=1:N
        lambda1=[lambda1;fminbnd(@(b) smooth_eight(b,X(:,i),F1,h,tau),-bound,bound)];
    end
    F0=F1;
   
    F1=[];
    for j=1:T
        F1=[F1;fminbnd(@(b) smooth_eight(b,X(j,:)',lambda1,h,tau),-bound,bound)];
    end
    m=m+1; % control the maximum number of iterations
end

F_dum=F1/sqrt(F1'*F1/T)*sign(F1(1,1));

lambda_dum=[];
     for i=1:N
         lambda_dum=[lambda_dum;fminbnd(@(b) smooth_eight(b,X(:,i),F_dum,h,tau),-bound,bound)];
     end;

     
%U_true=(epsi-norminv(tau)).*(F*ones(1,N));     
U = X - F_dum*lambda_dum';
u = U/b;

psi= ( 3/4*(1-u.^2 ) .* (abs(u)<1 )/b    )* (lambda_dum.^2)/N   ;
VAR_ESTIMATE =[VAR_ESTIMATE tau*(1-tau)* (lambda_dum'*lambda_dum/N)./(psi.^2)]  ; 
     
F_standard=[F_standard  sqrt(N)*(F_dum-F ) ];

end;

normal_approx=F_standard./(sqrt(VAR_ESTIMATE));


%% make the graphs in Figure 1 and Figure 2 of the online appendix

x=-4:0.01:4;
normal=1/sqrt(2*pi)*exp(-x.^2/2);

[bars,bins]=hist(normal_approx(T/2,:),16);
bar(bins,bars/trapz(bins,bars),'FaceColor',[0 0.4470 0.7410])
hold on;
plot(x,normal,'r')

xlim([-5 5])
ylim([0 0.45])
xticks([-5 0 5])
yticks([0 0.1 0.2 0.3 0.4])


